;   DAC Code Initialisation.
;   =======================
; In 3 parts :-
;	1. Initialise the Micro.
;	2. Initialise the Program.
;	3. Start the Interrupts.


ini_start
;	1. Initialise the Micro.
;	-----------------------
ini_micro
; Interrupts.
; ----------
	movlb	bank_1						; ( Bank 1.)
	clrf	OSCCON						; x4 PLL permanantly Enabled, HS clock determined by CONFIG FOSC<2:0> and PLLEN bits.
	clrf	INTCON
	movlw	b'01100111'					; All Weak Pullups Enabled, +ve INT edge Interrupt, TMR0 Clock to T0CKI (RA4) and +ve edge clocked,
	movwf	OPTION_REG					; Prescaler assigned to TMR0 with /256. ( TMR0 clock i/p is tied off and stopped.)
	clrf	PIE1						; Clear the Interrupt Enable registers,
	clrf	PIE2
	clrf	PIE3
	clrf	PIE4
	clrf	PIE5
	clrf	PIE6
	movlb	bank_0						; ( Bank 0.)
	clrf	PIR1						; and Interrupt flags.
	clrf	PIR2
	clrf	PIR3
	clrf	PIR4
	clrf	PIR5
	clrf	PIR6
	movlw	abs_hi_bank_0					; Clear FSR0H, FSR1H to Absolute Bank 0.
	movwf	FSR0H
	movwf	FSR1H

;	IOC Registers.
;	-------------						; Clear all IOC registers.
	movlb	bank_7						; ( Bank 7.)
	clrf	IOCAP						; Port A,
	clrf	IOCAN
	clrf	IOCAF
	clrf	IOCBP						; Port B,
	clrf	IOCBN
	clrf	IOCBF
	clrf	IOCCP						; Port C,
	clrf	IOCCN
	clrf	IOCCF
	clrf	IOCEP						; Port E.
	clrf	IOCEN
	clrf	IOCEF

;	Analogue Pins.
;	-------------
;	No pins are Analog inputs.
	movlb	bank_3						; ( Bank 3.)
	clrf	ANSELA
	clrf	ANSELB
	clrf	ANSELC

;	I/O Ports.
;	- - - - -
;	Inputs are defined as Digital CMOS Inputs with pullups.
;	Digital Outputs are push-pull with the output latches set low, and max. slew rate.
;	RA4 is TMR0's T0CKI clock i/p, pulled-up HIGH, unused.
;	Op. Amp. Outputs automatically control their output drivers.
	movlb	bank_1						; ( Bank 1.)
	movlw	B'11111111'
	movwf	TRISA
								; Bit  Lvl  Desc.
								;	7 - 1 - RA7, i/p. OSC1 \_ 8MHz crystal.
								;	6 - 1 - RA6, i/p. OSC2 /
								;	5 - 1 - RA5, i/p.
								;	4 - 1 - RA4, i/p. TMR0 T0CKI Clock I/p, unused.
								;	3 - 1 - RA3, i/p.
								;	2 - 1 - RA2, i/p.
								;	1 - 1 - RA1, i/p. Analog OPA1 o/p. ( DAC 1.)
								;	0 - 1 - RA0, i/p.
	movlb	bank_0						; ( Bank 0.)
	movlw	B'00000000'
	movwf	PORTA
								; Bit  Lvl  Desc.
								;	7 - 0 - RA7, i/p.
								;	6 - 0 - RA6, i/p.
								;	5 - 0 - RA5, i/p.
								;	4 - 0 - RA4, i/p.
								;	3 - 0 - RA3, i/p.
								;	2 - 0 - RA2, i/p.
								;	1 - 0 - RA1, i/p.
								;	0 - 0 - RA0, i/p.
	movlb	bank_2						; ( Bank 2.)
	movlw	B'00000000'
	movwf	LATA
								; Bit  Lvl  Desc.
								;	7 - 0 - RA7, i/p.
								;	6 - 0 - RA6, i/p.
								;	5 - 0 - RA5, i/p.
								;	4 - 0 - RA4, i/p.
								;	3 - 0 - RA3, i/p.
								;	2 - 0 - RA2, i/p.
								;	1 - 0 - RA1, i/p.
								;	0 - 0 - RA0, i/p.

	movlb	bank_4						; ( Bank 4.)
	movlw	b'11111111'					; All I/p pins have
	movwf	WPUA						; Enabled Pullups.
	movlb	bank_5						; ( Bank 5.)
	clrf	ODCONA						; Push-pull outputs.
	movlb	bank_6						; ( Bank 6.)
	clrf	SLRCONA						; Max. slew rate.
	movlb	bank_7						; ( Bank 7.)
	movlw	b'11111111'
	movwf	INLVLA						; All I/p levels CMOS.

	movlb	bank_1						; ( Bank 1.)
	movlw	B'11111111'
	movwf	TRISB
								; Bit  Lvl  Desc.
								;	7 - 1 - RB7, i/p.
								;	6 - 1 - RB6, i/p.
								;	5 - 1 - RB5, i/p.
								;	4 - 1 - RB4, i/p.
								;	3 - 1 - RB3, i/p.
								;	2 - 1 - RB2, i/p.
								;	1 - 1 - RB1, i/p. Analog OPA2 o/p. ( DAC 2.)
								;	0 - 1 - RB0, i/p.
	movlb	bank_0						; ( Bank 0.)
	movlw	B'00000000'
	movwf	PORTB
								; Bit  Lvl  Desc.
								;	7 - 0 - RB7, i/p.
								;	6 - 0 - RB6, i/p.
								;	5 - 0 - RB5, i/p.
								;	4 - 0 - RB4, i/p.
								;	3 - 0 - RB3, i/p.
								;	2 - 0 - RB2, i/p.
								;	1 - 0 - RB1, i/p.
								;	0 - 0 - RB0, i/p.
	movlb	bank_2						; ( Bank 2.)
	movlw	B'00000000'
	movwf	LATB
								; Bit  Lvl  Desc.
								;	7 - 0 - RB7, i/p.
								;	6 - 0 - RB6, i/p.
								;	5 - 0 - RB5, i/p.
								;	4 - 0 - RB4, i/p.
								;	3 - 0 - RB3, i/p.
								;	2 - 0 - RB2, i/p.
								;	1 - 0 - RB1, i/p.
								;	0 - 0 - RB0, i/p.

	movlb	bank_4						; ( Bank 4.)
	movlw	b'11111111'					; All I/p pins have
	movwf	WPUB						; Enabled Pullups.
	movlb	bank_5						; ( Bank 5.)
	clrf	ODCONB						; Push-pull outputs.
	movlb	bank_6						; ( Bank 6.)
	clrf	SLRCONB						; Max. slew rate.
	movlb	bank_7						; ( Bank 7.)
	movlw	b'11111111'
	movwf	INLVLB						; All I/p levels CMOS.
	movlb	bank_8						; ( Bank 8.)
	clrf	HIDRVB						; Standard O/p drive for RB<1-0>.

	movlb	bank_1						; ( Bank 1.)
	movlw	B'11110011'
	movwf	TRISC
								; Bit  Lvl  Desc.
								;	7 - 1 - RC7, i/p.
								;	6 - 1 - RC6, i/p.
								;	5 - 1 - RC5, i/p.
								;	4 - 1 - RC4, i/p.
								;	3 - 1 - RC3, o/p. DEBUG. PWM 5 o/p.
								;	2 - 1 - RC2, o/p. DEBUG. Interrupt toggling o/p.
								;	1 - 1 - RC1, i/p.
								;	0 - 0 - RC0, i/p.
	movlb	bank_0						; ( Bank 0.)
	movlw	B'00000000'
	movwf	PORTC
								; Bit  Lvl  Desc.
								;	7 - 0 - RC7, i/p.
								;	6 - 0 - RC6, i/p.
								;	5 - 0 - RC5, i/p.
								;	4 - 0 - RC4, i/p.
								;	3 - 0 - RC3, o/p.
								;	2 - 0 - RC2, i/p.
								;	1 - 0 - RC1, i/p.
								;	0 - 0 - RC0, i/p.
	movlb	bank_2						; ( Bank 2.)
	movlw	B'00000000'
	movwf	LATC
								; Bit  Lvl  Desc.
								;	7 - 0 - RC7, i/p.
								;	6 - 0 - RC6, i/p.
								;	5 - 0 - RC5, i/p.
								;	4 - 0 - RC4, i/p.
								;	3 - 0 - RC3, o/p.
								;	2 - 0 - RC2, i/p.
								;	1 - 0 - RC1, i/p.
								;	0 - 0 - RC0, i/p.

	movlb	bank_4						; ( Bank 4.)
	movlw	b'11111111'					; All I/p pins have
	movwf	WPUC						; Enabled Pullups.
	movlb	bank_5						; ( Bank 5.)
	clrf	ODCONC						; Push-pull outputs.
	movlb	bank_6						; ( Bank 6.)
	clrf	SLRCONC						; Max. slew rate.
	movlb	bank_7						; ( Bank 7.)
	clrf	INLVLC						; All I/p levels TTL.

	movlb	bank_4						; ( Bank 4.)
	bsf		WPUE,WPUE3				; MCLR pin, pulled-up.
	movlb	bank_7						; ( Bank 7.)
	movlw	b'00001000'					; DEBUG. I/p level CMOS.   !!!!! COMPILER FAILURE.  !!!!!
;	movwf	INLVLE						; DEBUG. Apparently MPLABX doesn't know this register exists in Bank 7, at h'390'.
	movwf	INLVE						; DEBUG. This IS defined as h'390'.

;	16-bit PWM 5.
;	------------
	movlb   bank_27						; ( Bank 27.)
	movlw	b'00000000'					; Disabled, active HIGH, Mode 1 - Standard PWM Mode,
	movwf	PWM5CON		
	movlw	b'00000000'					; Clock Prescale by 1, Fosc clock.
	movwf	PWM5CLKCON
	movlw	b'10000000'					; The "LDA" bit IS required for continuous output.
	movwf	PWM5LDCON			
	movlw	b'00000000'					; Independent Run Mode, no Offset Match or Trigger Source. 
	movwf	PWM5OFCON
	movlb	bank_29						; DEBUG. ( Bank 29.)
	movlw	b'000011101'					; DEBUG. PWM 5 o/p to pin RC3 via PPS.
	movwf	RC3PPS

;	10-bit DAC 1 and DAC 2.
;	----------------------
;	These use the Fixed Voltage Reference O/p 2.
;	DAC 1 and DAC 2 data are LEFT justified with the 8-bit samples going into the DACxREFH register.
;	The DACxREFL register is left clear. The o/p is fed internally to buffers OPA1 and OPA2 respectively,
;	set for unity gain, which have o/p's on RA1, pin 3 and RB1, pin 22.
	movlb   bank_1						; ( Bank 1.)
	movlw	b'10001000'					; Enable FVR_2 Buffer 2 o/p for DAC's, Gain x2.
	movwf	FVRCON
	movlb   bank_11						; ( Bank 11.)
	movlw	b'01001000'					; DAC 1 is Disabled, LEFT justified, no external O/p's
	movwf	DAC1CON0					; +ve source is FVR_2, -ve source is AVss. ( Vss.)
	movlw	b'01001000'					; DAC 2 is Disabled, LEFT justified, no external O/p's
	movwf	DAC2CON0					; +ve source is FVR_2, -ve source is AVss. ( Vss.)
	
;	OP. Amp's OPA1, OPA2.
;	--------------------
;	OPA1 is used to buffer the DAC 1 o/p. Its o/p pin is RA1, pin 3.
;	OPA2 is used to buffer the DAC 2 o/p. Its o/p pin is RB1, pin 22.
        movlb   bank_10						; ( Bank 10.)
	movlw	b'00000010'
	movwf	OPA1PCHS					; +ve i/p is from DAC 1,
	movlw	b'00000011'
	movwf	OPA2PCHS					; +ve i/p is from DAC 2.
	movlw	b'10010000'					; Both enabled, with Unity Gain Override.
	movwf	OPA1CON
	movwf	OPA2CON

;	2. Initialise the Program.
;	-------------------------
;	First the Sample Table and DAC 1 and 2 setup, followed by the timing source from PWM5.

ini_prog
	movlw	LOW wf_tbl_top					; Point to the Top of the DAC sample table,
	movwf   FSR0L						; and using registers in Bank 0, fill 11 registers
	clrf    FSR0H						; with the DAC values for a Sine wave.
	movlw   31						; +90 deg., "wf_tbl_top".
	movwi   FSR0++
	movlw   28						; +72 deg.
	movwi   FSR0++
	movlw	25						; +54 deg.
	movwi	FSR0++
	movlw	22						; +36 deg.
	movwi	FSR0++
	movlw	19						; +18 deg.
	movwi	FSR0++
	movlw	16						; 0 deg.,  "wf_tbl_mid".
	movwi	FSR0++
	movlw	13						; -18 deg.
	movwi	FSR0++
	movlw	10						; -36 deg.
	movwi	FSR0++
	movlw	7						; -54 deg.
	movwi	FSR0++
	movlw	4						; -72 deg.
	movwi	FSR0++
	movlw	1						; -90 deg., "wf_tbl_end".
	movwi	FSR0++
   	movlw	LOW wf_tbl_mid					; Set the FSR0 pointer to the Middle ( 0 deg.) of the DAC sample table.
	movwf   FSR0L
	movlb	bank_0						; ( Bank 0.)
	movlw	b'00000000'					; Set the direction flag to UP the sample table,
	movwf	flags						; ( Flags<0> = 0, Pointer is Decremented.)
	movf	INDF0,W						; get the sample value,
	
;	movlw	d'05'						; DEBUG. For the program memory sample table method.
;	movwf	table_pointer					; Set the table pointer to the middle sample.
;	call	sample_data					; Get the sample value,
 	
	movlb   bank_11						; ( Bank 11.)
	movwf	DAC1REFH					; and put into DAC 1. ( LEFT justified.)
	clrf	DAC1REFL					; ( DAC1REFL<7:6> 2 bits not used.)
	movwf	DAC2REFH					; Also put into  DAC 2. ( LEFT justified.)
	clrf	DAC2REFL					; ( DAC2REFL<7:6> 2 bits not used.)
	bsf	DACLD,DAC1LD					; Output the sample,
	bsf	DAC1CON0,DAC1EN					; and enable DAC 1 output.
	bsf	DACLD,DAC2LD					; Output the sample,
	bsf	DAC2CON0,DAC2EN					; and enable DAC 2 output.

	movlb   bank_27						; ( Bank 27.)
	clrf	PWM5PHH						; Clear the Phase,
	clrf	PWM5PHL
;	movlw	h'06'						; and set 50% Duty Cycle registers.
	movlw	HIGH 1600
	movwf	PWM5DCH						; Actual value is unimportant, just requires a sensible value.
;	movlw	h'40'						; ( h'0640' = 1600 clocks of Fosc, half the Period set below.)
	movlw	LOW 1600
	movwf	PWM5DCL
;	movlw	h'0C'						; Period timing, 3200 clock counts of Fosc, = h'0C80',
	movlw	HIGH 3200-1
	movwf	PWM5PRH						; or 500Hz with 20 samples/cycle. ( Fosc = 32MHz.)
;	movlw	h'7F'						; Period counter over-runs by 1 count, so h'0C7F' used.
	movlw	LOW 3200-1
	movwf	PWM5PRL

	clrf	PWM5OFH						; Clear the Offset register,
	clrf	PWM5OFL
	clrf	PWM5TMRH					; and the Timer.
	clrf	PWM5TMRL
	
;	3. Start the Interrupts.
;	-----------------------
;	PWM 5 Interrupt is Cleared and Enabled, then PWM 5 is started.

ini_int
	clrf	PWM5INTF					; Clear all Interrupt Request flags,
;	movlw	b'00000001'					; and Enable the Period Interrupt.
	movlw	(1<<PRIE)
	movwf	PWM5INTE
	movlb	bank_0						; ( Bank 0.)
	bcf	PIR6,PWM5IF					; Clear the PWM 5 group PIR6 flag,
	movlb   bank_1						; ( Bank 1.)
	bsf	PIE6,PWM5IE					; and Enable PWM 5 group PIE6 Interrupt,
	bsf	INTCON,PEIE					; the Peripheral and
	bsf	INTCON,GIE					; Global Enables.
	movlb   bank_27						; ( Bank 27.)
	bsf	PWM5CON,PWM5EN					; Start PWM 5.
	movlb	bank_0						; ( Bank 0.)

	return
